@@ -1,4 +1,5 @@
-siNMF <- function(X, M=NULL, initW=NULL, initH=NULL,
+siNMF <- function(X, M=NULL, pseudocount=1e-10,
+    initW=NULL, initH=NULL,
     fixW=FALSE, fixH=FALSE,
     L1_W=1e-10, L1_H=1e-10,
     L2_W=1e-10, L2_H=1e-10,
@@ -7,10 +8,11 @@ siNMF <- function(X, M=NULL, initW=NULL, initH=NULL,
     viz = FALSE, figdir = NULL, verbose = FALSE) {
     # Argument check
     algorithm <- match.arg(algorithm)
-    chk <- .checksiNMF(X, M, initW, initH, fixW, fixH, J, w,
+    chk <- .checksiNMF(X, M, pseudocount, initW, initH, fixW, fixH, J, w,
         p, thr, num.iter, viz, figdir, verbose)
     X <- chk$X
     M <- chk$M
+    pM <- chk$pM
     fixH <- chk$fixH
     w <- chk$w
     K <- chk$K
@@ -37,16 +39,16 @@ siNMF <- function(X, M=NULL, initW=NULL, initH=NULL,
             W_numer <- matrix(0, nrow=nrow(W), ncol=ncol(W))
             W_denom <- matrix(0, nrow=nrow(W), ncol=ncol(W))
             for(k in seq_len(K)){
-                W_numer <- W_numer + w[k] * (M[[k]] * X[[k]] * (M[[k]] * X_bar[[k]])^(-p)) %*% H[[k]]
-                W_denom <- W_denom + w[k] * (M[[k]] * W %*% t(H[[k]]))^(1-p) %*% H[[k]] + L1_W + L2_W * W
+                W_numer <- W_numer + w[k] * (pM[[k]] * X[[k]] * (pM[[k]] * X_bar[[k]])^(-p)) %*% H[[k]]
+                W_denom <- W_denom + w[k] * (pM[[k]] * W %*% t(H[[k]]))^(1-p) %*% H[[k]] + L1_W + L2_W * W
             }
             W <- .columnNorm(.positive(W * W_numer / W_denom))
         }
         # Update H_k
         for(k in seq_len(K)){
             if(!fixH[k]){
-                Hk_numer <- (t(M[[k]]*X[[k]]) * t(M[[k]]*X_bar[[k]])^(-p)) %*% W
-                Hk_denom <- t(M[[k]] * W %*% t(H[[k]]))^(1-p) %*% W + L1_H + L2_H * H[[k]]
+                Hk_numer <- (t(pM[[k]]*X[[k]]) * t(pM[[k]]*X_bar[[k]])^(-p)) %*% W
+                Hk_denom <- t(pM[[k]] * W %*% t(H[[k]]))^(1-p) %*% W + L1_H + L2_H * H[[k]]
                 H[[k]] <- H[[k]] * Hk_numer / Hk_denom
             }
         }
@@ -100,11 +102,9 @@ siNMF <- function(X, M=NULL, initW=NULL, initH=NULL,
         RelChange = RelChange))
 }
 
-.checksiNMF <- function(X, M, initW, initH, fixW, fixH, J, w,
+.checksiNMF <- function(X, M, pseudocount, initW, initH, fixW, fixH, J, w,
     p, thr, num.iter, viz, figdir, verbose){
-    if(!is.list(X)){
-        stop("input X must be specified as a list!")
-    }
+    stopifnot(is.list(X))
     if(length(X) < 2){
         stop("input list X must have at least two datasets!")
     }
@@ -120,6 +120,7 @@ siNMF <- function(X, M=NULL, initW=NULL, initH=NULL,
             M[[i]][] <- 1
         }
     }
+    stopifnot(is.numeric(pseudocount))
     if(!is.null(initW)){
         if(!identical(nrow(X[[1]]), nrow(initW))){
             stop("Please specify nrow(X[[1]]) and nrow(initW) same")
@@ -132,9 +133,7 @@ siNMF <- function(X, M=NULL, initW=NULL, initH=NULL,
             stop("Please specify all the ncol(initH[[k]]) are same as ncol(X[[k]]) (k=1,2,...)")
         }
     }
-    if(!is.logical(fixW)){
-        stop("Please specify the fixW as a logical")
-    }
+    stopifnot(is.logical(fixW))
     if(!is.logical(fixH)){
         if(!is.vector(fixH)){
             stop("Please specify the fixH as a logical or a logical vector such as c(TRUE, FALSE, TRUE)")
@@ -146,10 +145,7 @@ siNMF <- function(X, M=NULL, initW=NULL, initH=NULL,
     }else{
         fixH <- rep(fixH, length=length(X))
     }
-
-    if(!is.numeric(J)){
-        stop("Please specify J as numeric!")
-    }
+    stopifnot(is.numeric(J))
     if(is.null(w)){
         w <- rep(1, length=length(X))
     }else{
@@ -159,26 +155,21 @@ siNMF <- function(X, M=NULL, initW=NULL, initH=NULL,
             w <- w / sum(w)
         }
     }
-    if(!is.numeric(p)){
-        stop("Please specify p as numeric!")
-    }
-    if(!is.numeric(thr)){
-        stop("Please specify thr as numeric!")
-    }
-    if(!is.numeric(num.iter)){
-        stop("Please specify num.iter as numeric!")
-    }
-    if(!is.logical(viz)){
-        stop("Please specify the viz as a logical")
-    }
+    stopifnot(is.numeric(p))
+    stopifnot(is.numeric(thr))
+    stopifnot(is.numeric(num.iter))
+    stopifnot(is.logical(viz))
+    stopifnot(is.logical(verbose))
     if(!is.character(figdir) && !is.null(figdir)){
         stop("Please specify the figdir as a string or NULL")
     }
-    if(!is.logical(verbose)){
-        stop("Please specify the verbose as a logical")
-    }
+    pM <- M
+    lapply(seq_along(X), function(x){
+         X[[x]][which(X[[x]] == 0)] <<- pseudocount
+         pM[[x]][which(pM[[x]] == 0)] <<- pseudocount
+     })
     K <- length(X)
-    list(X=X, M=M, w=w, fixH=fixH, K=K)
+    list(X=X, M=M, pM=pM, w=w, fixH=fixH, K=K)
 }
 
 .initsiNMF <- function(X, initW, initH, J, p, thr, algorithm, verbose){
