@@ -12,6 +12,12 @@ out9 <- NMF(X, J=3, algorithm="HALS", num.iter=2)
 out10 <- NMF(X, J=3, algorithm="PGD", num.iter=2)
 out11 <- NMF(X, J=3, algorithm="GCD", num.iter=2)
 
+out12 <- NMF(X, J=3, algorithm="Projected", num.iter=2)
+out13 <- NMF(X, J=3, algorithm="NHR", num.iter=2)
+out14 <- NMF(X, J=3, algorithm="DTPP", num.iter=2)
+out15 <- NMF(X, J=3, algorithm="Orthogonal", num.iter=2)
+out16 <- NMF(X, J=3, algorithm="OrthReg", num.iter=2)
+
 expect_equivalent(length(out1), 10)
 expect_equivalent(length(out2), 10)
 expect_equivalent(length(out3), 10)
@@ -23,3 +29,8 @@ expect_equivalent(length(out8), 10)
 expect_equivalent(length(out9), 10)
 expect_equivalent(length(out10), 10)
 expect_equivalent(length(out11), 10)
+expect_equivalent(length(out12), 10)
+expect_equivalent(length(out13), 10)
+expect_equivalent(length(out14), 10)
+expect_equivalent(length(out15), 10)
+expect_equivalent(length(out16), 10)
